unit Query;

interface

uses
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, ExtCtrls, Db, Grids, DBGrids, StdCtrls, ToolWin, ComCtrls, Buttons,
{$IFNDEF FPC}
  MemDS,
{$ELSE}
  MemDataSet,
{$ENDIF}
  LiteDACVcl, DemoFrame, LiteDacDemoForm, DBAccess, LiteAccess;

type
  TQueryFrame = class(TDemoFrame)
    DataSource: TDataSource;
    LiteQuery: TLiteQuery;
    DBGrid: TDBGrid;
    Panel9: TPanel;
    btRefreshRecord: TSpeedButton;
    DBNavigator: TDBNavigator;
    Splitter1: TSplitter;
    ToolBar: TPanel;
    Panel1: TPanel;
    btClose: TSpeedButton;
    btOpen: TSpeedButton;
    Panel2: TPanel;
    cbRefreshBeforeEdit: TCheckBox;
    cbRefreshAfterInsert: TCheckBox;
    cbRefreshAfterUpdate: TCheckBox;
    Panel5: TPanel;
    StaticText1: TLabel;
    edFetchRows: TEdit;
    Panel6: TPanel;
    Label5: TLabel;
    edFilter: TEdit;
    cbFiltered: TCheckBox;
    Panel7: TPanel;
    edUpdatingTable: TEdit;
    Label2: TLabel;
    Memo: TMemo;
    btPrepare: TSpeedButton;
    btUnPrepare: TSpeedButton;
    btExecute: TSpeedButton;
    btSaveToXML: TSpeedButton;
    SaveDialog: TSaveDialog;
    Panel3: TPanel;
    Label1: TLabel;
    cmbLockMode: TComboBox;
    cbFetchAll: TCheckBox;
    Panel4: TPanel;
    btLockRecord: TSpeedButton;
    btUnLock: TSpeedButton;
    procedure btOpenClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btRefreshRecordClick(Sender: TObject);
    procedure DataSourceDataChange(Sender: TObject; Field: TField);
    procedure DataSourceStateChange(Sender: TObject);
    procedure cbFilteredClick(Sender: TObject);
    procedure cbRefreshOptionsClick(Sender: TObject);
    procedure LiteQueryAfterOpen(DataSet: TDataSet);
    procedure edFetchRowsExit(Sender: TObject);
    procedure btPrepareClick(Sender: TObject);
    procedure btUnPrepareClick(Sender: TObject);
    procedure btExecuteClick(Sender: TObject);
    procedure btSaveToXMLClick(Sender: TObject);
    procedure btLockRecordClick(Sender: TObject);
    procedure cbDMLRefreshClick(Sender: TObject);
    procedure btUnLockClick(Sender: TObject);
    procedure cbFetchAllClick(Sender: TObject);
    procedure edUpdatingTableExit(Sender: TObject);
    procedure cmbLockModeChange(Sender: TObject);
    procedure edFilterExit(Sender: TObject);
  private
    { Private declarations }
    procedure AssignProperties(PropertyNameToSet: string = '');
    procedure ShowState;
  public
    // Demo management
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

procedure TQueryFrame.ShowState;
var
  St: string;

  procedure AddSt(S:string);
  begin
    if St <> '' then
      St := St + ', ';
    St := St + S;
  end;

begin
  St := '';

  if LiteQuery.Prepared then begin
    AddSt('Prepared');

    if LiteQuery.IsQuery then
      AddSt('IsQuery');
  end;

  if LiteQuery.Active then
    AddSt('Active')
  else
    AddSt('Inactive');

  if LiteQuery.Executing then
    AddSt('Executing');

  if LiteQuery.Fetching then
    AddSt('Fetching');

  edUpdatingTable.Text := LiteQuery.UpdatingTable;

  LiteDacForm.StatusBar.Panels[2].Text := St;
end;

procedure TQueryFrame.AssignProperties(PropertyNameToSet: string = '');  // empty string means all properties
begin
  if CheckProperty('FetchRows', PropertyNameToSet) then
    try
      LiteQuery.FetchRows := StrToInt(edFetchRows.Text);
    except
      edFetchRows.SetFocus;
      raise;
    end;

  if CheckProperty('FetchAll', PropertyNameToSet) then
    try
      LiteQuery.FetchAll := cbFetchAll.Checked;
    finally
      cbFetchAll.Checked := LiteQuery.FetchAll;
    end;

  if CheckProperty('UpdatingTable', PropertyNameToSet) then
    try
      LiteQuery.UpdatingTable := edUpdatingTable.Text;
    except
      edUpdatingTable.SetFocus;
      raise;
    end;

  if CheckProperty('Filter', PropertyNameToSet) then
    LiteQuery.Filter := edFilter.Text;

  if CheckProperty('Filtered', PropertyNameToSet) then
    try
      LiteQuery.Filtered := cbFiltered.Checked;
    finally
      cbFiltered.Checked := LiteQuery.Filtered;
    end;

  if CheckProperty('SQL', PropertyNameToSet) then
    if Trim(LiteQuery.SQL.Text) <> Trim(Memo.Lines.Text) then
      LiteQuery.SQL.Assign(Memo.Lines);

  if CheckProperty('RefreshOptions', PropertyNameToSet) then begin
    if cbRefreshBeforeEdit.Checked then
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions + [roBeforeEdit]
    else
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions - [roBeforeEdit];
    if cbRefreshAfterInsert.Checked then
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions + [roAfterInsert]
    else
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions - [roAfterInsert];
    if cbRefreshAfterUpdate.Checked then
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions + [roAfterUpdate]
    else
      LiteQuery.RefreshOptions := LiteQuery.RefreshOptions - [roAfterUpdate];
  end;

  if CheckProperty('LockMode', PropertyNameToSet) then
    case cmbLockMode.ItemIndex of
      1:
        LiteQuery.LockMode := lmOptimistic;
      2:
        LiteQuery.LockMode := lmPessimistic;
    else
      LiteQuery.LockMode := lmNone;
    end;
end;

procedure TQueryFrame.btOpenClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteQuery.Open;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btCloseClick(Sender: TObject);
begin
  LiteQuery.Close;
  ShowState;
end;

procedure TQueryFrame.btRefreshRecordClick(Sender: TObject);
begin
  LiteQuery.RefreshRecord;
end;

procedure TQueryFrame.DataSourceStateChange(Sender: TObject);
begin
  LiteDacForm.StatusBar.Panels[1].Text := 'Record ' + IntToStr(LiteQuery.RecNo) + ' of ' + IntToStr(LiteQuery.RecordCount);
end;

procedure TQueryFrame.DataSourceDataChange(Sender: TObject; Field: TField);
begin
  DataSourceStateChange(nil);
end;

procedure TQueryFrame.cbFilteredClick(Sender: TObject);
begin
  AssignProperties('Filtered');
end;

procedure TQueryFrame.edFilterExit(Sender: TObject);
begin
  AssignProperties('Filter');
end;

procedure TQueryFrame.cbRefreshOptionsClick(Sender: TObject);
begin
  AssignProperties('RefreshOptions');
end;

procedure TQueryFrame.cbDMLRefreshClick(Sender: TObject);
begin
  AssignProperties('DMLRefresh');
end;

procedure TQueryFrame.cbFetchAllClick(Sender: TObject);
begin
  AssignProperties('FetchAll');
end;

procedure TQueryFrame.edFetchRowsExit(Sender: TObject);
begin
  AssignProperties('FetchRows');
end;

procedure TQueryFrame.edUpdatingTableExit(Sender: TObject);
begin
  AssignProperties('UpdatingTable');
end;

procedure TQueryFrame.cmbLockModeChange(Sender: TObject);
begin
  AssignProperties('LockMode');
end;

procedure TQueryFrame.LiteQueryAfterOpen(DataSet: TDataSet);
begin
  ShowState;
end;

procedure TQueryFrame.btPrepareClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteQuery.Prepare;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btUnPrepareClick(Sender: TObject);
begin
  LiteQuery.UnPrepare;
  ShowState;
end;

procedure TQueryFrame.btExecuteClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteQuery.Execute;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btSaveToXMLClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    LiteQuery.SaveToXML(SaveDialog.FileName);
end;

procedure TQueryFrame.btLockRecordClick(Sender: TObject);
begin
  LiteQuery.Lock;
end;

procedure TQueryFrame.btUnLockClick(Sender: TObject);
begin
  LiteQuery.UnLock;
end;

// Demo management
procedure TQueryFrame.Initialize;
begin
  inherited;
  LiteQuery.Connection := Connection as TLiteConnection;
  edFetchRows.Text := IntToStr(LiteQuery.FetchRows);

  cbRefreshBeforeEdit.Checked := roBeforeEdit in LiteQuery.RefreshOptions;
  cbRefreshAfterInsert.Checked := roAfterInsert in LiteQuery.RefreshOptions;
  cbRefreshAfterUpdate.Checked := roAfterUpdate in LiteQuery.RefreshOptions;

  edFilter.Text := LiteQuery.Filter;
  cbFiltered.Checked := LiteQuery.Filtered;

  Memo.Lines.Text := LiteQuery.SQL.Text;
end;

procedure TQueryFrame.SetDebug(Value: boolean);
begin
  LiteQuery.Debug := Value;
end;

{$IFDEF FPC}
initialization
  {$i Query.lrs}
{$ENDIF}

end.

